<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://polkurier.pl
 * @since      1.0.0
 *
 * @package    Polkurier
 * @subpackage Polkurier/admin
 */

use Polkurier\Http\Request;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Polkurier
 * @subpackage Polkurier/admin
 * @author     Polkurier <api@polkurier.pl>
 */
class Polkurier_Admin
{

    private $option_name = 'polkurier';
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     * @since    1.0.0
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        add_filter('woocommerce_admin_order_actions', function ($actions, $order) {
            $orderStatus = $order->get_status();
            if ($orderStatus !== 'pending' && $orderStatus !== 'on-hold' && $orderStatus !== 'completed' && $orderStatus !== 'cancelled' && $orderStatus !== 'refunded' && $orderStatus !== 'failed') {
                $status = isset($_GET['status']) ? $_GET['status'] : null;
                $order_id = method_exists($order, 'get_id') ? $order->get_id() : $order->id;
                require_once dirname(__FILE__) . '/../includes/PolkurierAPI.php';
                $order = PolkurierOrder::getOrderByWCOrderId($order_id);
                if (!$order) {
                    $shipButtonUrl = wp_nonce_url(admin_url('admin.php?page=polkurier_packages&action=create&order_id=' . $order_id));
                    $actions['polkurier'] = array(
                        'url' => wp_nonce_url($shipButtonUrl, 'polkurier'),
                        'name' => __('Zamów usługę Polkurier', 'woocommerce'),
                        'action' => 'polkurier polkurier-action-button',
                    );
                    $actions['polkurier-quick'] = array(
                        'url' => $shipButtonUrl,
                        'name' => __('Szybie zamówienie przesyłki', 'woocommerce'),
                        'action' => 'polkurier-quick polkurier-action-button polkurier-prevent-default',
                    );
                } else {
                    $actions['polkurier-print-label'] = array(
                        'url' => wp_nonce_url(admin_url('admin.php?page=polkurier_packages&action=label&label=' . $order->order_number)),
                        'name' => __('Drukuj etykietę', 'woocommerce'),
                        'action' => 'polkurier wc-action-button-polkurier-print-label polkurier-action-button polkurier-open-in-new-tab',
                    );
                    $actions['polkurier-print-protocol'] = array(
                        'url' => wp_nonce_url(admin_url('admin.php?page=polkurier_packages&action=protocol&label=' . $order->order_number)),
                        'name' => __('Drukuj protokół', 'woocommerce'),
                        'action' => 'polkurier wc-action-button-polkurier-print-protocol polkurier-action-button polkurier-open-in-new-tab',
                    );
                }
            }
            return $actions;
        }, 100, 2);

        $getShopOrderId = static function ($shopOrder) {
            global $post;
            if ($shopOrder instanceof \WC_Order) {
                return $shopOrder->get_id();
            }
            if ($post !== null) {
                return $post->ID;
            }
            return null;
        };

        $trackingOrderColumn = static function ($column_name, $shopOrder = null) {
            global $post;

            if ($shopOrder instanceof \WC_Order) {
                $orderId = $shopOrder->get_id();
            } else if ($post !== null) {
                $orderId = $post->ID;
            } else {
                $orderId = null;
            }

            if ($column_name === 'package_number') {
                $polkurier = Polkurier::instance();
                wc_get_template('/OrdersList/package_number_column.php', [
                    'shopOrderId' => $orderId,
                    'order' => $orderId ? PolkurierOrder::getOrderByWCOrderId($orderId) : null,
                ], '', $polkurier->getPluginPath('templates'));
            }
        };

        add_filter('manage_shop_order_posts_custom_column', $trackingOrderColumn);
        add_action('manage_woocommerce_page_wc-orders_custom_column', $trackingOrderColumn, 10, 2);

        add_action('admin_head', function() {
            $polkurier = \Polkurier::instance();
            wc_get_template(
                '/scripts/admin_global_js.php',
                [
                    'ajaxNonce' => wp_create_nonce('polkurier-nonce'),
                    'ajaxUrl' =>  admin_url('admin-ajax.php'),
                    'esmUrl' => $polkurier->getPluginUrl(),
                ],
                '',
                $polkurier->getPluginPath('templates')
            );
        });

        add_filter('wp_script_attributes', function ($attributes) {
            if (isset($attributes['id']) && stripos($attributes['id'], '--module') !== false) {
                $attributes['type'] = 'module';
            }
            return $attributes;
        }, 1, 1);

        add_action('admin_enqueue_scripts', function() {
            $polkurier = Polkurier::instance();
            wp_enqueue_script('pk-orders-list--module', $polkurier->getPluginUrl() . 'admin/js/OrdersList/index.js?ver=1.26.2', [], $polkurier->get_version());
        });

        add_action('add_meta_boxes', static function ($shopOrder) use ($getShopOrderId) {
            $polkurier = Polkurier::instance();
            $shopOrderId = $getShopOrderId($shopOrder);
            add_meta_box($polkurier->get_plugin_name() . '_delivery', 'Polkurier.pl', static function () use ($polkurier, $shopOrderId) {
                wc_get_template('/OrderView/metabox.php', [
                    'shopOrderId' => $shopOrderId,
                    'order' => $shopOrderId ? PolkurierOrder::getOrderByWCOrderId($shopOrderId) : null,
                ], '', $polkurier->getPluginPath('templates'));
            }, wc_get_page_screen_id('shop-order'), 'side', 'high' );
        });

        function add_polkurier_column_notice()
        {
            ?>
            <div id="polkurier-column-notice" class="notice notice-error is-dismissible hidden">
                <p><?php _e('<b>Uwaga</b>, aby skorzystać z możliwości wtyczki Polkurier, włącz kolumnę "Działania" w menu <b>opcje ekranu</b> w prawym górnym narożniku tego widoku. <b>Bez włączenia tej kolumny nie będzie możliwe zamawianie przesyłek przez usługę Polkurier.</b>', 'sample-text-domain'); ?></p>
            </div>
            <script type="text/javascript">
                jQuery(document).ready(function ($) {
                    if ($(".column-wc_actions").hasClass("hidden")) {
                        $("#polkurier-column-notice").removeClass("hidden");
                    }
                });
            </script>
            <?php
        }

        if (isset($_GET['post_type']) && $_GET['post_type'] == 'shop_order') {
            add_action('admin_notices', 'add_polkurier_column_notice');
        }
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Polkurier_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Polkurier_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style(uniqid('pk', true), plugin_dir_url(__FILE__) . 'css/select2.min.css', array(), $this->version, 'all');
        wp_enqueue_style(uniqid('pk', true), plugin_dir_url( __FILE__ ) . 'css/pk-utils.css', array(), $this->version, 'all' );
        wp_enqueue_style(uniqid('pk', true), plugin_dir_url( __FILE__ ) . 'css/polkurier-admin.css', array(), $this->version, 'all' );
        wp_enqueue_style(uniqid('pk', true), 'https://maps.polkurier.pl/assets/dist/points-map.css', [], $this->version);
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Polkurier_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Polkurier_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script(uniqid('pk', true), plugin_dir_url(__FILE__) . 'js/polkurier-admin.js?ver=1.26.2', ['jquery'], $this->version, false);
        wp_enqueue_script(uniqid('pk', true), 'https://maps.polkurier.pl/assets/dist/points-map.bundle.js?ver=1.26.2', ['jquery'], $this->version,true);

    }

    // Create the Plugin Name menu page with add_menu_page();
    public function add_admin_page()
    {
        if (!class_exists('WooCommerce')) {
            add_action('admin_notices', array($this, 'no_woocommerce_notice'));
        } else {
            add_submenu_page(
                'woocommerce',
                'Ustawienia Polkurier',
                'Ustawienia Polkurier',
                'manage_options',
                $this->plugin_name,
                array($this, 'load_admin_page_content') // Calls function to require the partial
            );
            add_submenu_page(
                'woocommerce',
                'Przesyłki',
                'Przesyłki',
                'manage_woocommerce',
                $this->plugin_name . '_packages',
                array($this, 'load_admin_page_packages') // Calls function to require the partial
            );
            add_submenu_page(
                'woocommerce',
                'Wycena',
                'Wycena',
                'manage_woocommerce',
                $this->plugin_name . '_valuation',
                array($this, 'load_admin_page_valuation') // Calls function to require the partial
            );
        }
        if (!get_option('polkurier_apikey') || !get_option('polkurier_account_id')) {
            add_action('admin_notices', array($this, 'no_settings_notice'));
        }
    }

    // Load the plugin admin page partial.
    public function load_admin_page_content()
    {
        require_once plugin_dir_path(__FILE__) . 'partials/polkurier-admin-display.php';
    }

    public function load_admin_page_valuation()
    {
        require_once plugin_dir_path(__FILE__) . 'partials/polkurier-admin-valuation.php';
    }

    public function load_admin_page_packages()
    {
        require_once plugin_dir_path(__FILE__) . 'partials/polkurier-admin-packages.php';
    }

    public function register_setting()
    {
        add_settings_section(
            $this->option_name . '_authorization',
            __('Autoryzacja', $this->plugin_name),
            array($this, $this->option_name . '_authorize_cb'),
            $this->plugin_name . '_authorization'
        );
        add_settings_section(
            $this->option_name . '_address',
            __('Domyślne adresy', $this->plugin_name),
            array($this, $this->option_name . '_address_cb'),
            $this->plugin_name . '_address'
        );
        add_settings_section(
            $this->option_name . '_accounts',
            __('Konta bankowe', $this->plugin_name),
            array($this, $this->option_name . '_accounts_cb'),
            $this->plugin_name . '_accounts'
        );
        add_settings_section(
            $this->option_name . '_templates',
            __('Szablony przesyłek', $this->plugin_name),
            array($this, $this->option_name . '_templates_cb'),
            $this->plugin_name . '_templates'
        );
        add_settings_section(
            $this->option_name . '_default_carrier',
            __('Domyślny przewoźnik', $this->plugin_name),
            array($this, $this->option_name . '_default_carrier_section_cb'),
            $this->plugin_name . '_default_carrier'
        );

        add_settings_field(
            $this->option_name . '_account_id',
            __('ID konta', $this->plugin_name),
            array($this, $this->option_name . '_account_id_cb'),
            $this->plugin_name . '_authorization',
            $this->option_name . '_authorization',
            array('label_for' => $this->option_name . '_account_id')
        );
        add_settings_field(
            $this->option_name . '_apikey',
            __('Token', $this->plugin_name),
            array($this, $this->option_name . '_apikey_cb'),
            $this->plugin_name . '_authorization',
            $this->option_name . '_authorization',
            array('label_for' => $this->option_name . '_apikey')
        );

        add_settings_field(
            $this->option_name . '_company_name',
            __('Nazwa firmy', $this->plugin_name),
            array($this, $this->option_name . '_company_name_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_company_name')
        );
        add_settings_field(
            $this->option_name . '_name_surname',
            __('Imię i nazwisko', $this->plugin_name),
            array($this, $this->option_name . '_name_surname_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_name_surname_cb')
        );
        add_settings_field(
            $this->option_name . '_street',
            __('Ulica', $this->plugin_name),
            array($this, $this->option_name . '_street_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_street_cb')
        );
        add_settings_field(
            $this->option_name . '_number_1',
            __('Numer budynku', $this->plugin_name),
            array($this, $this->option_name . '_number_1_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_number_1_cb')
        );
        add_settings_field(
            $this->option_name . '_number_2',
            __('Numer lokalu', $this->plugin_name),
            array($this, $this->option_name . '_number_2_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_number_2_cb')
        );
        add_settings_field(
            $this->option_name . '_postal_code',
            __('Kod pocztowy', $this->plugin_name),
            array($this, $this->option_name . '_postal_code_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_postal_code_cb')
        );
        add_settings_field(
            $this->option_name . '_city',
            __('Miasto', $this->plugin_name),
            array($this, $this->option_name . '_city_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_city_cb')
        );
        add_settings_field(
            $this->option_name . '_phone',
            __('Numer telefonu', $this->plugin_name),
            array($this, $this->option_name . '_phone_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_phone_cb')
        );
        add_settings_field(
            $this->option_name . '_email',
            __('Adres e-mail', $this->plugin_name),
            array($this, $this->option_name . '_email_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_email_cb')
        );
        add_settings_field(
            $this->option_name . '_polkurier_account_id',
            __('ID konta Polkurier', $this->plugin_name),
            array($this, $this->option_name . '_polkurier_account_id_cb'),
            $this->plugin_name . '_address',
            $this->option_name . '_address',
            array('label_for' => $this->option_name . '_polkurier_account_id_cb')
        );

        add_settings_field(
            $this->option_name . '_account_number',
            __('Numer konta bankowego', $this->plugin_name),
            array($this, $this->option_name . '_account_number_cb'),
            $this->plugin_name . '_accounts',
            $this->option_name . '_accounts',
            array('label_for' => $this->option_name . '_account_number_cb')
        );

        add_settings_field(
            $this->option_name . '_default_carrier_id',
            __('Domyślny przewoźnik', $this->plugin_name),
            array($this, $this->option_name . '_default_carrier_cb'),
            $this->plugin_name . '_default_carrier',
            $this->option_name . '_default_carrier',
            array('label_for' => $this->option_name . '_default_carrier_cb')
        );

        add_settings_field(
            $this->option_name . '_templates',
            __('Szablony paczek', $this->plugin_name),
            array($this, $this->option_name . '_templates_cb'),
            $this->plugin_name . '_templates',
            $this->option_name . '_templates',
            array('label_for' => $this->option_name . '_templates_cb')
        );

        add_settings_field(
            $this->option_name . '_adresses',
            __('Domyślne adresy', $this->plugin_name),
            array($this, $this->option_name . '_templates_cb'),
            $this->plugin_name . '_adresses',
            $this->option_name . '_adresses',
            array('label_for' => $this->option_name . '_templates_cb')
        );

        register_setting($this->plugin_name . '_authorization', $this->option_name . '_apikey', ['type' => 'string']);
        register_setting($this->plugin_name . '_authorization', $this->option_name . '_account_id', ['type' => 'string']);

        register_setting($this->plugin_name . '_address', $this->option_name . '_company_name', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_name_surname', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_street', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_number_1', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_number_2', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_postal_code', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_city', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_phone', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_email', ['type' => 'string']);
        register_setting($this->plugin_name . '_address', $this->option_name . '_polkurier_account_id', ['type' => 'string']);

        register_setting($this->plugin_name . '_address', $this->option_name . '_adresses', ['type' => 'string']);

        register_setting($this->plugin_name . '_accounts', $this->option_name . '_accounts', ['type' => 'string']);

        register_setting($this->plugin_name . '_templates', $this->option_name . '_templates', ['type' => 'string']);

        register_setting($this->plugin_name . '_default_carrier', $this->option_name . '_default_carrier_id', ['type' => 'string']);
    }

    public function polkurier_default_carrier_cb()
    {
        $option_value = get_option($this->option_name . '_default_carrier_id');
        $api = $this->get_polkurier_api();
        foreach ($api->getAvailableCarriers() as $carrier) {
            echo '<input id="polkurier_default_carrier_id_' . $carrier["servicecode"] . '" type="radio" name="polkurier_default_carrier_id" value="' . $carrier["servicecode"] . '" ' . checked($carrier["servicecode"], $option_value, false) . ' />';
            echo '<label for="polkurier_default_carrier_id_' . $carrier["servicecode"] . '">' . $carrier['name'] . '</label><br/>';
        }
    }

    public function polkurier_authorize_cb()
    {
        echo '<p>' . __('Ustawienia autoryzacji', $this->plugin_name) . '</p>';
    }

    public function polkurier_default_carrier_section_cb()
    {
        echo '<p>Ustawienia domyślnego przewoźnika usługi Polkurier</p>';
    }

    public function polkurier_address_cb()
    {
        echo '<p>Ustawienia adresowe nadawcy</p>';
    }

    protected function polkurier_generic_cb($fieldName, $type = 'text')
    {
        $option_value = get_option($this->option_name . $fieldName);
        echo '<input type="' . $type . '" name="' . $this->option_name . $fieldName . '" id="' . $this->option_name . $fieldName . '" value="' . $option_value . '" > ';
    }

    public function polkurier_account_number_cb()
    {
        return $this->polkurier_generic_cb('_accounts');
    }

    public function polkurier_account_id_cb()
    {
        return $this->polkurier_generic_cb('_account_id');
    }

    public function polkurier_apikey_cb()
    {
        return $this->polkurier_generic_cb('_apikey');
    }

    public function polkurier_company_name_cb()
    {
        return $this->polkurier_generic_cb('_company_name');
    }

    public function polkurier_name_surname_cb()
    {
        return $this->polkurier_generic_cb('_name_surname');
    }

    public function polkurier_street_cb()
    {
        return $this->polkurier_generic_cb('_street');
    }

    public function polkurier_number_1_cb()
    {
        return $this->polkurier_generic_cb('_number_1');
    }

    public function polkurier_number_2_cb()
    {
        return $this->polkurier_generic_cb('_number_2');
    }

    public function polkurier_postal_code_cb()
    {
        return $this->polkurier_generic_cb('_postal_code');
    }

    public function polkurier_city_cb()
    {
        return $this->polkurier_generic_cb('_city');
    }

    public function polkurier_phone_cb()
    {
        return $this->polkurier_generic_cb('_phone');
    }

    public function polkurier_email_cb()
    {
        return $this->polkurier_generic_cb('_email');
    }

    public function polkurier_polkurier_account_id_cb()
    {
        return $this->polkurier_generic_cb('_polkurier_account_id');
    }

//    public function polkurier_display_table()
//    {
//        require_once plugin_dir_path(__FILE__) . '../includes/Polkurier_WP_List_Table.php';
//        $table = new Polkurier_WP_List_Table();
//        $table->prepare_items();
//        $table->display();
//    }

    public function no_woocommerce_notice()
    {
        echo '<div class="notice error polkurier-notice is-dismissible" ><p>Błąd Polkurier: nie znaleziono WooCommerce. Sprawdź konfigurację swojej instancji Wordpress.</p></div>';
    }

    public function no_settings_notice()
    {
        echo '<div class="notice error polkurier-notice is-dismissible" ><p>Aby użyć wtyczki Polkurier, wprowadź wszystkie <a href="admin.php?page=polkurier">ustawienia</a>.</p></div>';
    }

    public static function api_error_notice($msg)
    {
        echo '<div class="notice error polkurier-notice is-dismissible" ><p>' . $msg . '</p></div>';
    }

    protected function get_polkurier_api()
    {
        require_once plugin_dir_path(__FILE__) . '../includes/PolkurierAPI.php';
        return new PolkurierAPI(get_option('polkurier_account_id'), get_option('polkurier_apikey'));
    }

    public function get_package_details()
    {
        if (isset($_GET['package'])) {
            $api = $this->get_polkurier_api();
            return $api->getPackageDetails($_GET['package']);
        } else {
            return array();
        }

    }

    protected function describe_field($field, $value)
    {
        $desc = '';
        switch ($field) {
            case 'orderNumber':
                $desc = 'Numer zamówienia';
                break;
            case 'size':
                $desc = 'Rozmiar paczki';
                break;
            case 'status':
                $desc = 'Status';
                break;
            case 'name':
                $desc = 'Imię';
                break;
            case 'surname':
                $desc = 'Nazwisko';
                break;
            case 'street':
                $desc = 'Ulica';
                break;
            case 'number':
                $desc = 'Adres';
                break;
            case 'zipCode':
                $desc = 'Kod pocztowy';
                break;
            case 'city':
                $desc = 'Miasto';
                break;
            case 'companyName':
                $desc = 'Nazwa firmy';
                break;
            default:
                return false;
        }
        return "<p><b>$desc:</b> $value</p>";
    }

}
