<?php

namespace Polkurier\Controller;


use DateTime;
use Exception;
use Polkurier\ApiClient;
use Polkurier\Exception\HttpException;
use Polkurier\Http\JsonResponse;
use Polkurier\Http\Request;
use Polkurier\Util\Arr;
use Polkurier\Util\ParameterBag;
use PolkurierAddress;
use PolkurierOrder;

class OrderController
{

    /**
     * Wycena zamówienia
     * @param Request $request
     * @return JsonResponse
     * @throws HttpException
     */
    public function evaluateAction(Request $request)
    {

        $params = [
            "shipmenttype" => $request->getRequest('polkurier_type'),
            "packs" => [],
            'COD' => (float)$request->getRequest('polkurier_pobranie', 0.0),
            'codtype' => $request->getRequest('polkurier_pobranie_zwrot', ''),
            'return_cod' => $request->getRequest('polkurier_pobranie_rodzaj', ''),
            'insurance' => (float)$request->getRequest('polkurier_ubezpieczenie', 0.0),
            'postcode_recipient' => '',
            'postcode_sender' => '',
            'recipient_country' => '',
            'courierservice' => (array)$request->getRequest('courierservice', []),
        ];

        $packagesList = json_decode(stripslashes($request->getRequest('packages_list', '[]')), true);
        foreach ($packagesList as $package) {
            $params['packs'][] = [
                "length" => (float)$package['depth'],
                "width" => (float)$package['width'],
                "height" => (float)$package['height'],
                "weight" => (float)$package['weight'],
                "amount" => (int)$package['count'],
                "type" => $package['nonstandard'] ? "NST" : "ST",
            ];
        }

        $orderId = (int)$request->getRequest('order_id');
        if ($orderId > 0) {
            $order = wc_get_order($orderId);
            if ($order !== false) {
                $orderData = $order->get_data();
                $params['recipient_country'] = $orderData['shipping']['country'];
                $params['postcode_recipient'] = $orderData['shipping']['postcode'];
            }
        }

        $senderAddress = PolkurierAddress::getAddressById((int)$request->getRequest('polkurier_address'));
        if (!empty($senderAddress)) {
            $params["postcode_sender"] = $senderAddress->address['postcode'];
        }

        if (empty($params['packs']) || empty($params['shipmenttype'])) {
            return new JsonResponse([
                'success' => false,
            ]);
        }

        $recipient = (array)$request->getRequest('recipient', []);
        $recipientCountry = Arr::get($recipient, 'country', '');
        if ($recipientCountry !== '') {
            $params['recipient_country'] = $recipientCountry;
        }
        $coverAddress = $request->getRequest('polkurier_cover_address');

        if ($coverAddress !== 'null') {
            $params['courierservice'] = [
                'COVER_ADDRESS_SENDER' => 1
            ];
        }

        try {
            $response = ApiClient::getInstance()->makeApiRequest('order_valuation', $params);
            return new JsonResponse([
                'success' => true,
                'data' => $response,
            ]);
        } catch (Exception $e) {
            throw new HttpException($e->getMessage(), 403);
        }
    }

    /**
     * Wycena zamówienia
     * @param Request $request
     * @return JsonResponse
     * @throws HttpException
     */
    public function createAction(Request $request)
    {
        try {
            $props = $request->request;
            $recipient = new ParameterBag($props->getArray('recipient'));
            $sender = new ParameterBag($props->getArray('sender'));
            $pickup = new ParameterBag($props->getArray('pickup'));
            $cod = new ParameterBag($props->getArray('cod'));

            $order = [
                'platform' => 2,
                'courier' => $props->get('courier'),
                'shipmenttype' => $props->get('shipmenttype'),
                'description' => $props->get('description'),
                'sender' => $props->getArray('sender'),
                'recipient' => $props->getArray('recipient'),
                'packs' => array_map(static function ($pack) {
                    return [
                        'length' => (float)Arr::get($pack, 'length'),
                        'width' => (float)Arr::get($pack, 'width'),
                        'height' => (float)Arr::get($pack, 'height'),
                        'weight' => (float)Arr::get($pack, 'weight'),
                        'amount' => (int)Arr::get($pack, 'amount'),
                        'type' => Arr::get($pack, 'type', 'ST'),
                    ];
                }, $props->getArray('packs')),
                'pickup' => [
                    'pickupdate' => $pickup->get('pickupdate'),
                    'pickuptimefrom' => $pickup->get('pickuptimefrom'),
                    'pickuptimeto' => $pickup->get('pickuptimeto'),
                    'nocourierorder' => $pickup->getBoolean('nocourierorder'),
                ],
                'COD' => [
                    'codtype' => (string)$cod->get('codtype'),
                    'codamount' => (string)$cod->get('codamount'),
                    'codbankaccount' => (string)$cod->get('codbankaccount'),
                    'return_cod' => (string)$cod->get('return_cod'),
                ],
                'insurance' => $props->get('insurance'),
                'courierservice' => [],
                'cover_address' => $props->getArray('cover_address'),
            ];

            foreach ($props->getArray('courierservice') as $service => $enabled) {
                if ($enabled === true || $enabled === 'true') {
                    $order['courierservice'][$service] = true;
                }
            }

            $response = ApiClient::getInstance()->makeApiRequest('create_order', $order);

            $order = new PolkurierOrder();
            $order->order_number = (string)$response['order_number'];
            $order->label = (array)$response['label'];
            $order->price_gross = (float)$response['price_gross'];
            $order->price_net = (float)$response['price_net'];
            $order->carrier = (string)$props->get('courier');
            $order->address = $sender->all();
            $order->address_to = $recipient->all();
            $order->pobranie = $cod->getFloat('codamount');
            $order->extra = [];
            $order->date_created = new DateTime();
            $order->wp_order_id = $props->getInt('order_id');
            $order->save();

            return new JsonResponse([
                'order_id' => $props->getInt('order_id'),
                'order_number' => (string)$response['order_number'],
                'label' => (array)$response['label'],
                'price_gross' => (float)$response['price_gross'],
                'price_net' => (float)$response['price_net'],
            ]);
        } catch (Exception $e) {
            throw new HttpException($e->getMessage(), 403);
        }
    }

    /**
     * Wycena zamówienia
     * @param Request $request
     * @return JsonResponse
     * @throws HttpException
     */
    public function evaluateV2Action(Request $request)
    {
        $props = $request->request;
        $recipient = new ParameterBag($props->getArray('recipient'));
        $sender = new ParameterBag($props->getArray('sender'));
        $cod = new ParameterBag($props->getArray('cod'));

        $params = [
            "shipmenttype" => $props->get('shipmenttype'),
            'COD' => $cod->getFloat('codamount'),
            'codtype' => $cod->get('codtype', ''),
            'return_cod' => $cod->get('return_cod', ''),
            'insurance' => $props->getFloat('insurance'),
            'postcode_recipient' => $recipient->get('postcode'),
            'recipient_country' => $recipient->get('country'),
            'postcode_sender' => $sender->get('postcode'),
            'sender_country' => $sender->get('country'),
            'courierservice' => $props->getArray('courierservice'),
            'packs' => $props->getArray('packs'),
        ];

        try {
            return new JsonResponse(ApiClient::getInstance()->makeApiRequest('order_valuation', $params));
        } catch (Exception $e) {
            throw new HttpException($e->getMessage(), 403);
        }
    }

}
