<?php
namespace Polkurier\Order;

use InvalidArgumentException;
use WC_Order;

class OrderShippingPointManager
{

    private static $TABLE_NAME = 'polkurier_shipping_points';
    private static $ORDER_POINT_CACHE = [];

    /**
     * @param WC_Order|int $orderOrId
     * @return int
     */
    private function ensureOrderId($orderOrId)
    {
        $orderId = (int)($orderOrId instanceof WC_Order ? $orderOrId->get_id() : $orderOrId);
        if ($orderId <= 0) {
            return 0;
        }
        $id = wp_is_post_revision($orderId);
        return $id !== false ? $id : $orderId;
    }

    /**
     * @param WC_Order|int $orderOrId
     * @param ShippingPoint $point
     * @return ShippingPoint
     */
    public function save($orderOrId, $point)
    {
        global $wpdb;

        if ($point instanceof ShippingPoint === false) {
            throw new InvalidArgumentException('$point must be of ShippingPoint class');
        }

        $orderId = $this->ensureOrderId($orderOrId);
        if ($orderId <= 0) {
            throw new InvalidArgumentException('Invalid order ID');
        }


        $previousPoint = $this->getOrderShippingPointInstance($orderId);
        if ($previousPoint !== null && !empty($previousPoint->id)) {
            $wpdb->update($wpdb->prefix . self::$TABLE_NAME, [
                'wp_order_id' => $orderId,
                'code' => $point->code,
                'label' => $point->label,
                'provider' => $point->provider,
                'type' => $point->type,
            ], [
                'id' => $previousPoint->id,
            ]);
        } else {
            $wpdb->insert($wpdb->prefix . self::$TABLE_NAME, [
                'id' => null,
                'wp_order_id' => $orderId,
                'code' => $point->code,
                'label' => $point->label,
                'provider' => $point->provider,
                'type' => $point->type,
            ]);
            $point->id = $wpdb->insert_id;
        }
        self::$ORDER_POINT_CACHE[$orderOrId] = $point;
        return $point;
    }

    /**
     * @param WC_Order|int $orderOrId
     * @return ShippingPoint|null
     */
    private function getOrderShippingPointInstance($orderId)
    {
        global $wpdb;

        if ($orderId <= 0) {
            return null;
        }

        if (self::$ORDER_POINT_CACHE[$orderId] !== null) {
            return self::$ORDER_POINT_CACHE[$orderId];
        }

        $tableName = $wpdb->prefix . self::$TABLE_NAME;
        $data = $wpdb->get_results( "SELECT * FROM `{$tableName}` WHERE `wp_order_id` = $orderId LIMIT 1");
        if (count($data) === 1 && (int)$data[0]->id > 0) {
            $point = new ShippingPoint();
            $point->id = (int)$data[0]->id;
            $point->wp_order_id = $orderId;
            $point->code = (string)$data[0]->code;
            $point->label = (string)$data[0]->label;
            $point->provider = (string)$data[0]->provider;
            $point->type = (string)$data[0]->type;

            self::$ORDER_POINT_CACHE[$orderId] = $point;
            return $point;
        }
        return null;
    }

    /**
     * @param WC_Order|int $orderOrId
     * @return ShippingPoint|null
     */
    public function get($orderOrId)
    {
        global $wpdb;

        $orderId = $this->ensureOrderId($orderOrId);
        if ($orderId <= 0) {
            return null;
        }

        if (self::$ORDER_POINT_CACHE[$orderId] !== null) {
            return self::$ORDER_POINT_CACHE[$orderId];
        }

        $point = $this->getOrderShippingPointInstance($orderId);
        if ($point !== null) {
            return $point;
        }

        $point = new ShippingPoint();

        // Legacy - pobiera z postmeta
        $point->wp_order_id = $orderId;
        $point->code = (string)get_post_meta($orderId, '_polkurier_point_id', true);
        $point->provider = (string)get_post_meta($orderId, '_polkurier_provider', true);
        $point->label = (string)get_post_meta($orderId, '_polkurier_point_label', true);
        $point->type = (string)get_post_meta($orderId, '_polkurier_point_type', true);

        if (empty($point->code)) {
            $meta = $wpdb->get_results( "SELECT * FROM `{$wpdb->prefix}postmeta` WHERE `post_id` = $orderId");
            foreach ($meta as $entry) {
                switch ($entry->meta_key) {
                    case '_polkurier_point_id':
                        $point->code = (string)$entry->meta_value;
                        break;
                    case '_polkurier_provider':
                        $point->provider = (string)$entry->meta_value;
                        break;
                    case '_polkurier_point_label':
                        $point->label = (string)$entry->meta_value;
                        break;
                    case '_polkurier_point_type':
                        $point->type = (string)$entry->meta_value;
                        break;
                }
            }
        }

        // Dane o punkcie odbioru ze wtyczki "Inpost Paczkomaty" by Damian Ziarnik
        // @see https://pl.wordpress.org/plugins/inpost-paczkomaty/
        if (empty($point->code)) {
            $point->code = (string)get_post_meta($orderId, 'paczkomat_key', true);
            $point->label = (string)get_post_meta($orderId, 'Wybrany paczkomat', true);
            if (!empty($point->code)) {
                $point->provider = 'INPOST_PACZKOMAT';
            }
        }

        if (!empty($point->code)) {
            self::$ORDER_POINT_CACHE[$orderId] = $point;
            return $point;
        }

        return null;
    }

}
